@@ -16,14 +16,15 @@
 
 import copy
 import os
+import shutil
 import tempfile
+from urllib import parse as urlparse
 
 from ironic_lib import utils as ironic_utils
 import jinja2
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import excutils
-from oslo_utils import fileutils
 
 from ironic.common import dhcp_factory
 from ironic.common import exception
@@ -64,13 +65,16 @@ KERNEL_RAMDISK_LABELS = {'deploy': DEPLOY_KERNEL_RAMDISK_LABELS,
                          'rescue': RESCUE_KERNEL_RAMDISK_LABELS}
 
 
-def get_root_dir():
-    """Returns the directory where the config files and images will live."""
-    return CONF.pxe.tftp_root
+def _get_root_dir(ipxe_enabled):
+    if ipxe_enabled:
+        return CONF.deploy.http_root
+    else:
+        return CONF.pxe.tftp_root
 
 
-def get_ipxe_root_dir():
-    return CONF.deploy.http_root
+def ensure_tree(path):
+    mode = CONF.pxe.dir_permission or 0o755
+    os.makedirs(path, mode=mode, exist_ok=True)
 
 
 def _ensure_config_dirs_exist(task, ipxe_enabled=False):
@@ -80,10 +84,7 @@ def _ensure_config_dirs_exist(task, ipxe_enabled=False):
     :param ipxe_enabled: Default false boolean to indicate if ipxe
                          is in use by the caller.
     """
-    if ipxe_enabled:
-        root_dir = get_ipxe_root_dir()
-    else:
-        root_dir = get_root_dir()
+    root_dir = _get_root_dir(ipxe_enabled)
     node_dir = os.path.join(root_dir, task.node.uuid)
     pxe_dir = os.path.join(root_dir, PXE_CFG_DIR_NAME)
     # NOTE: We should only change the permissions if the folder
@@ -92,9 +93,7 @@ def _ensure_config_dirs_exist(task, ipxe_enabled=False):
     # which may not recurse downward.
     for directory in (node_dir, pxe_dir):
         if not os.path.isdir(directory):
-            fileutils.ensure_tree(directory)
-            if CONF.pxe.dir_permission:
-                os.chmod(directory, CONF.pxe.dir_permission)
+            ensure_tree(directory)
 
 
 def _link_mac_pxe_configs(task, ipxe_enabled=False):
@@ -163,7 +162,7 @@ def _link_ip_address_pxe_configs(task, ipxe_enabled=False):
 
 
 def _get_pxe_grub_mac_path(mac, ipxe_enabled=False):
-    root_dir = get_ipxe_root_dir() if ipxe_enabled else get_root_dir()
+    root_dir = _get_root_dir(ipxe_enabled)
     yield os.path.join(root_dir, "%s-%s-%s" %
                        ("grub.cfg", "01", mac.replace(':', "-").lower()))
     yield os.path.join(root_dir, mac + '.conf')
@@ -188,9 +187,9 @@ def _get_pxe_mac_path(mac, delimiter='-', client_id=None,
         if client_id:
             hw_type = '20-'
         mac_file_name = hw_type + mac_file_name
-        return os.path.join(get_root_dir(), PXE_CFG_DIR_NAME,
+        return os.path.join(CONF.pxe.tftp_root, PXE_CFG_DIR_NAME,
                             mac_file_name)
-    return os.path.join(get_ipxe_root_dir(), PXE_CFG_DIR_NAME,
+    return os.path.join(CONF.deploy.http_root, PXE_CFG_DIR_NAME,
                         mac_file_name)
 
 
@@ -225,10 +224,7 @@ def get_kernel_ramdisk_info(node_uuid, driver_info, mode='deploy',
 
     Note: driver_info should be validated outside of this method.
     """
-    if ipxe_enabled:
-        root_dir = get_ipxe_root_dir()
-    else:
-        root_dir = get_root_dir()
+    root_dir = _get_root_dir(ipxe_enabled)
     image_info = {}
     labels = KERNEL_RAMDISK_LABELS[mode]
     for label in labels:
@@ -248,10 +244,7 @@ def get_pxe_config_file_path(node_uuid, ipxe_enabled=False):
     :returns: The path to the node's PXE configuration file.
 
     """
-    if ipxe_enabled:
-        return os.path.join(get_ipxe_root_dir(), node_uuid, 'config')
-    else:
-        return os.path.join(get_root_dir(), node_uuid, 'config')
+    return os.path.join(_get_root_dir(ipxe_enabled), node_uuid, 'config')
 
 
 def get_file_path_from_label(node_uuid, root_dir, label):
@@ -268,11 +261,12 @@ def get_file_path_from_label(node_uuid, root_dir, label):
     :param label: Name of the image
     """
     if label == 'ks_template':
-        return os.path.join(get_ipxe_root_dir(), node_uuid, 'ks.cfg.template')
+        return os.path.join(CONF.deploy.http_root, node_uuid,
+                            'ks.cfg.template')
     elif label == 'ks_cfg':
-        return os.path.join(get_ipxe_root_dir(), node_uuid, 'ks.cfg')
+        return os.path.join(CONF.deploy.http_root, node_uuid, 'ks.cfg')
     elif label == 'stage2':
-        return os.path.join(get_ipxe_root_dir(), node_uuid, 'LiveOS',
+        return os.path.join(CONF.deploy.http_root, node_uuid, 'LiveOS',
                             'squashfs.img')
     else:
         return os.path.join(root_dir, node_uuid, label)
@@ -378,7 +372,8 @@ def create_ipxe_boot_script():
     """Render the iPXE boot script into the HTTP root directory"""
     boot_script = utils.render_template(
         CONF.pxe.ipxe_boot_script,
-        {'ipxe_for_mac_uri': PXE_CFG_DIR_NAME + '/'})
+        {'ipxe_for_mac_uri': PXE_CFG_DIR_NAME + '/',
+         'ipxe_fallback_script': CONF.pxe.ipxe_fallback_script})
     bootfile_path = os.path.join(
         CONF.deploy.http_root,
         os.path.basename(CONF.pxe.ipxe_boot_script))
@@ -425,12 +420,8 @@ def clean_up_pxe_config(task, ipxe_enabled=False):
         for path in _get_pxe_grub_mac_path(port.address,
                                            ipxe_enabled=ipxe_enabled):
             ironic_utils.unlink_without_raise(path)
-    if ipxe_enabled:
-        utils.rmtree_without_raise(os.path.join(get_ipxe_root_dir(),
-                                                task.node.uuid))
-    else:
-        utils.rmtree_without_raise(os.path.join(get_root_dir(),
-                                                task.node.uuid))
+    utils.rmtree_without_raise(os.path.join(_get_root_dir(ipxe_enabled),
+                                            task.node.uuid))
 
 
 def _dhcp_option_file_or_url(task, urlboot=False, ip_version=None):
@@ -566,9 +557,11 @@ def dhcp_options_for_instance(task, ipxe_enabled=False, url_boot=False,
         # from it's own path, but Petitboot needs it to be specified by this
         # option since it doesn't use pxelinux.0 loader.
         if not url_boot:
+            # Enforce trailing slash
+            prefix = os.path.join(CONF.pxe.tftp_root, '')
             dhcp_opts.append(
                 {'opt_name': DHCP_TFTP_PATH_PREFIX,
-                 'opt_value': get_tftp_path_prefix()})
+                 'opt_value': prefix})
 
     if not url_boot:
         dhcp_opts.append({'opt_name': DHCP_TFTP_SERVER_NAME,
@@ -599,23 +592,6 @@ def dhcp_options_for_instance(task, ipxe_enabled=False, url_boot=False,
     return dhcp_opts
 
 
-def get_tftp_path_prefix():
-    """Adds trailing slash (if needed) necessary for path-prefix
-
-    :return: CONF.pxe.tftp_root ensured to have a trailing slash
-    """
-    return os.path.join(CONF.pxe.tftp_root, '')
-
-
-def get_path_relative_to_tftp_root(file_path):
-    """Return file relative path to CONF.pxe.tftp_root
-
-    :param file_path: full file path to be made relative path.
-    :returns: The path relative to CONF.pxe.tftp_root
-    """
-    return os.path.relpath(file_path, get_tftp_path_prefix())
-
-
 def is_ipxe_enabled(task):
     """Return true if ipxe is set.
 
@@ -680,10 +656,7 @@ def get_instance_image_info(task, ipxe_enabled=False):
     if (node.driver_internal_info.get('is_whole_disk_image')
             or deploy_utils.get_boot_option(node) == 'local'):
         return image_info
-    if ipxe_enabled:
-        root_dir = get_ipxe_root_dir()
-    else:
-        root_dir = get_root_dir()
+    root_dir = _get_root_dir(ipxe_enabled)
     i_info = node.instance_info
     if i_info.get('boot_iso'):
         image_info['boot_iso'] = (
@@ -692,47 +665,68 @@ def get_instance_image_info(task, ipxe_enabled=False):
 
         return image_info
 
-    labels = ('kernel', 'ramdisk')
+    image_properties = None
     d_info = deploy_utils.get_image_instance_info(node)
+
+    def _get_image_properties():
+        nonlocal image_properties
+        if not image_properties:
+            glance_service = service.GlanceImageService(context=ctx)
+            image_properties = glance_service.show(
+                d_info['image_source'])['properties']
+
+    labels = ('kernel', 'ramdisk')
     if not (i_info.get('kernel') and i_info.get('ramdisk')):
-        glance_service = service.GlanceImageService(context=ctx)
-        iproperties = glance_service.show(d_info['image_source'])['properties']
+        # NOTE(rloo): If both are not specified in instance_info
+        # we won't use any of them. We'll use the values specified
+        # with the image, which we assume have been set.
+        _get_image_properties()
         for label in labels:
-            i_info[label] = str(iproperties[label + '_id'])
+            i_info[label] = str(image_properties[label + '_id'])
         node.instance_info = i_info
         node.save()
 
     anaconda_labels = ()
     if deploy_utils.get_boot_option(node) == 'kickstart':
-        # stage2  - Installer stage2 squashfs image
-        # ks_template - Anaconda kickstart template
+        # stage2: installer stage2 squashfs image
+        # ks_template: anaconda kickstart template
         # ks_cfg - rendered ks_template
         anaconda_labels = ('stage2', 'ks_template', 'ks_cfg')
-        if not (i_info.get('stage2') and i_info.get('ks_template')):
-            iproperties = glance_service.show(
-                d_info['image_source']
-            )['properties']
-            for label in anaconda_labels:
-                # ks_template is an optional property on the image
-                if (label == 'ks_template'
-                        and not iproperties.get('ks_template')):
-                    i_info[label] = CONF.anaconda.default_ks_template
-                elif label == 'ks_cfg':
-                    i_info[label] = ''
-                elif label == 'stage2' and 'stage2_id' not in iproperties:
-                    msg = ("stage2_id property missing on the image. "
-                           "The anaconda deploy interface requires stage2_id "
-                           "property to be associated with the os image. ")
+        # NOTE(rloo): We save stage2 & ks_template values in case they
+        # are changed by the user after we start using them and to
+        # prevent re-computing them again.
+        if not node.driver_internal_info.get('stage2'):
+            if i_info.get('stage2'):
+                node.set_driver_internal_info('stage2', i_info['stage2'])
+            else:
+                _get_image_properties()
+                if 'stage2_id' not in image_properties:
+                    msg = (_("'stage2_id' is missing from the properties of "
+                             "the OS image %s. The anaconda deploy interface "
+                             "requires this to be set with the OS image or "
+                             "in instance_info['stage2']. ") %
+                           d_info['image_source'])
                     raise exception.ImageUnacceptable(msg)
                 else:
-                    i_info[label] = str(iproperties['stage2_id'])
-
-            node.instance_info = i_info
+                    node.set_driver_internal_info(
+                        'stage2', str(image_properties['stage2_id']))
+            if i_info.get('ks_template'):
+                node.set_driver_internal_info('ks_template',
+                                              i_info['ks_template'])
+            else:
+                _get_image_properties()
+                # ks_template is an optional property on the image
+                if 'ks_template' not in image_properties:
+                    node.set_driver_internal_info(
+                        'ks_template', CONF.anaconda.default_ks_template)
+                else:
+                    node.set_driver_internal_info(
+                        'ks_template', str(image_properties['ks_template']))
             node.save()
 
     for label in labels + anaconda_labels:
         image_info[label] = (
-            i_info[label],
+            i_info.get(label) or node.driver_internal_info.get(label, ''),
             get_file_path_from_label(node.uuid, root_dir, label)
         )
 
@@ -769,6 +763,7 @@ def build_deploy_pxe_options(task, pxe_info, mode='deploy',
     node = task.node
     kernel_label = '%s_kernel' % mode
     ramdisk_label = '%s_ramdisk' % mode
+    initrd_filename = ramdisk_label
     for label, option in ((kernel_label, 'deployment_aki_path'),
                           (ramdisk_label, 'deployment_ari_path')):
         if ipxe_enabled:
@@ -777,14 +772,18 @@ def build_deploy_pxe_options(task, pxe_info, mode='deploy',
                     and service_utils.is_glance_image(image_href)):
                 pxe_opts[option] = images.get_temp_url_for_glance_image(
                     task.context, image_href)
+                if label == ramdisk_label:
+                    path = urlparse.urlparse(pxe_opts[option]).path.strip('/')
+                    if path:
+                        initrd_filename = path.split('/')[-1]
             else:
                 pxe_opts[option] = '/'.join([CONF.deploy.http_url, node.uuid,
                                             label])
         else:
-            pxe_opts[option] = get_path_relative_to_tftp_root(
-                pxe_info[label][1])
+            pxe_opts[option] = os.path.relpath(pxe_info[label][1],
+                                               CONF.pxe.tftp_root)
     if ipxe_enabled:
-        pxe_opts['initrd_filename'] = ramdisk_label
+        pxe_opts['initrd_filename'] = initrd_filename
     return pxe_opts
 
 
@@ -810,8 +809,8 @@ def build_instance_pxe_options(task, pxe_info, ipxe_enabled=False):
                 # image_source to determine if it's a whole disk image or not.
                 # For example, when transitioning to 'available' state
                 # for first time from 'manage' state.
-                pxe_opts[option] = get_path_relative_to_tftp_root(
-                    pxe_info[label][1])
+                pxe_opts[option] = os.path.relpath(pxe_info[label][1],
+                                                   CONF.pxe.tftp_root)
 
     pxe_opts.setdefault('aki_path', 'no_kernel')
     pxe_opts.setdefault('ari_path', 'no_ramdisk')
@@ -941,14 +940,6 @@ def build_service_pxe_config(task, instance_image_info,
         ipxe_enabled=ipxe_enabled, anaconda_boot=anaconda_boot)
 
 
-def _build_heartbeat_url(node_uuid):
-
-    api_version = 'v1'
-    heartbeat_api = '%s/heartbeat/{node_uuid}' % api_version
-    path = heartbeat_api.format(node_uuid=node_uuid)
-    return "/".join([deploy_utils.get_ironic_api_url(), path])
-
-
 def build_kickstart_config_options(task):
     """Build the kickstart template options for a node
 
@@ -968,7 +959,13 @@ def build_kickstart_config_options(task):
     node.save()
     params['liveimg_url'] = node.instance_info['image_url']
     params['agent_token'] = node.driver_internal_info['agent_secret_token']
-    params['heartbeat_url'] = _build_heartbeat_url(node.uuid)
+
+    heartbeat_url = '%s/v1/heartbeat/%s' % (
+        deploy_utils.get_ironic_api_url().rstrip('/'),
+        node.uuid
+    )
+    params['heartbeat_url'] = heartbeat_url
+    params['config_drive'] = ks_utils.prepare_config_drive(task)
     return {'ks_options': params}
 
 
@@ -1079,6 +1076,7 @@ def validate_kickstart_template(ks_template):
     """
     ks_options = {'liveimg_url': 'fake_image_url',
                   'agent_token': 'fake_token',
+                  'config_drive': '',
                   'heartbeat_url': 'fake_heartbeat_url'}
     params = {'ks_options': ks_options}
     try:
@@ -1118,16 +1116,17 @@ def validate_kickstart_file(ks_cfg):
         return
 
     with tempfile.NamedTemporaryFile(
-            dir=CONF.tempdir, suffix='.cfg') as ks_file:
-        ks_file.writelines(ks_cfg)
+            dir=CONF.tempdir, suffix='.cfg', mode='wt') as ks_file:
+        ks_file.write(ks_cfg)
+        ks_file.flush()
         try:
-            result = utils.execute(
+            utils.execute(
                 'ksvalidator', ks_file.name, check_on_exit=[0], attempts=1
             )
-        except processutils.ProcessExecutionError:
-            msg = _(("The kickstart file generated does not pass validation. "
-                     "The ksvalidator tool returned following error(s): %s") %
-                    (result))
+        except processutils.ProcessExecutionError as e:
+            msg = (_("The kickstart file generated does not pass validation. "
+                     "The ksvalidator tool returned the following error: %s") %
+                   e)
             raise exception.InvalidKickstartFile(msg)
 
 
@@ -1196,9 +1195,6 @@ def prepare_instance_kickstart_config(task, image_info, anaconda_boot=False):
     ks_options = build_kickstart_config_options(task)
     kickstart_template = image_info['ks_template'][1]
     ks_cfg = utils.render_template(kickstart_template, ks_options)
-    ks_config_drive = ks_utils.prepare_config_drive(task)
-    if ks_config_drive:
-        ks_cfg = ks_cfg + ks_config_drive
     utils.write_to_file(image_info['ks_cfg'][1], ks_cfg)
 
 
@@ -1220,21 +1216,18 @@ def cache_ramdisk_kernel(task, pxe_info, ipxe_enabled=False):
     node = task.node
     t_pxe_info = copy.copy(pxe_info)
     if ipxe_enabled:
-        path = os.path.join(get_ipxe_root_dir(), node.uuid)
+        path = os.path.join(CONF.deploy.http_root, node.uuid)
     else:
-        path = os.path.join(get_root_dir(), node.uuid)
-    fileutils.ensure_tree(path)
-    # anconda deploy will have 'stage2' as one of the labels in pxe_info dict
-    if 'stage2' in pxe_info.keys():
-        # stage2  will be stored in ipxe http directory. So make sure they
-        # exist.
-        fileutils.ensure_tree(
-            get_file_path_from_label(
-                node.uuid,
-                get_ipxe_root_dir(),
-                'stage2'
-            )
-        )
+        path = os.path.join(CONF.pxe.tftp_root, node.uuid)
+    ensure_tree(path)
+    # anaconda deploy will have 'stage2' as one of the labels in pxe_info dict
+    if 'stage2' in pxe_info:
+        # stage2 will be stored in ipxe http directory so make sure the
+        # directory exists.
+        file_path = get_file_path_from_label(node.uuid,
+                                             CONF.deploy.http_root,
+                                             'stage2')
+        ensure_tree(os.path.dirname(file_path))
         # ks_cfg is rendered later by the driver using ks_template. It cannot
         # be fetched and cached.
         t_pxe_info.pop('ks_cfg')
@@ -1262,3 +1255,51 @@ def clean_up_pxe_env(task, images_info, ipxe_enabled=False):
 
     clean_up_pxe_config(task, ipxe_enabled=ipxe_enabled)
     TFTPImageCache().clean_up()
+
+
+def place_loaders_for_boot(base_path):
+    """Place configured bootloaders from the host OS.
+
+    Example: grubaa64.efi:/path/to/grub-aarch64.efi,...
+
+    :param base_path: Destination path where files should be copied to.
+    """
+    loaders = CONF.pxe.loader_file_paths
+    if not loaders or not base_path:
+        # Do nothing, return.
+        return
+
+    for dest, src in loaders.items():
+        (head, _tail) = os.path.split(dest)
+        if head:
+            if head.startswith('/'):
+                # NOTE(TheJulia): The intent here is to error if the operator
+                # has put absolute paths in place, as we can and likely should
+                # copy to multiple folders based upon the protocol operation
+                # being used. Absolute paths are problematic there, where
+                # as a relative path is more a "well, that is silly, but okay
+                # misconfiguration.
+                msg = ('File paths configured for [pxe]loader_file_paths '
+                       'must be relative paths. Entry: %s') % dest
+                raise exception.IncorrectConfiguration(msg)
+            else:
+                try:
+                    ensure_tree(os.path.join(base_path, head))
+                except OSError as e:
+                    msg = ('Failed to create supplied directories in '
+                           'asset copy paths. Error: %s') % e
+                    raise exception.IncorrectConfiguration(msg)
+
+        full_dest = os.path.join(base_path, dest)
+        LOG.debug('Copying bootloader %(dest)s from %(src)s.',
+                  {'src': src, 'dest': full_dest})
+        try:
+            shutil.copy2(src, full_dest)
+            if CONF.pxe.file_permission:
+                os.chmod(full_dest, CONF.pxe.file_permission)
+        except OSError as e:
+            msg = ('Error encountered while attempting to '
+                   'copy a configured bootloader into '
+                   'the requested destination. %s' % e)
+            LOG.error(msg)
+            raise exception.IncorrectConfiguration(error=msg)
