@@ -608,7 +608,7 @@ def get_boot_option(node):
     :raises: InvalidParameterValue if the capabilities string is not a
          dict or is malformed.
     :returns: A string representing the boot option type. Defaults to
-        'netboot'.
+        configuration setting [deploy]default_boot_mode.
     """
 
     # NOTE(TheJulia): Software raid always implies local deployment
@@ -628,7 +628,7 @@ def get_boot_option(node):
 # option it supports.
 
 def is_ramdisk_deploy(node):
-    return node.deploy_interface == 'ramdisk'
+    return node.get_interface('deploy') == 'ramdisk'
 
 
 def is_anaconda_deploy(node):
@@ -638,7 +638,7 @@ def is_anaconda_deploy(node):
     :returns: A boolean value of True when Anaconda deploy interface is in use
               otherwise False
     """
-    if node.deploy_interface == 'anaconda':
+    if node.get_interface('deploy') == 'anaconda':
         return True
     return False
 
@@ -721,8 +721,7 @@ def prepare_inband_cleaning(task, manage_boot=True):
     agent_add_clean_params(task)
 
     if manage_boot:
-        ramdisk_opts = build_agent_options(task.node)
-        task.driver.boot.prepare_ramdisk(task, ramdisk_opts)
+        prepare_agent_boot(task)
 
     # NOTE(dtantsur): calling prepare_ramdisk may power off the node, so we
     # need to check fast-track again and reboot if needed.
@@ -1406,6 +1405,27 @@ def set_async_step_flags(node, reboot=None, skip_current_step=None,
     node.save()
 
 
+def prepare_agent_boot(task):
+    """Prepare booting the agent on the node.
+
+    :param task: a TaskManager instance.
+    """
+    deploy_opts = build_agent_options(task.node)
+    task.driver.boot.prepare_ramdisk(task, deploy_opts)
+
+
+def reboot_to_finish_step(task):
+    """Reboot the node into IPA to finish a deploy/clean step.
+
+    :param task: a TaskManager instance.
+    :returns: states.CLEANWAIT if cleaning operation in progress
+              or states.DEPLOYWAIT if deploy operation in progress.
+    """
+    prepare_agent_boot(task)
+    manager_utils.node_power_action(task, states.REBOOT)
+    return get_async_step_return_state(task.node)
+
+
 def get_root_device_for_deploy(node):
     """Get a root device requested for deployment or None.
 
