@@ -186,6 +186,20 @@ _FIRMWARE_UPDATE_SUM_ARGSINFO = {
     }
 }
 
+_CLEAR_CA_CERTS_ARGSINFO = {
+    'certificate_files': {
+        'description': (
+            "The list of files containing the certificates to be cleared. "
+            "If empty list is specified, all the certificates on the ilo "
+            "will be cleared, except the certificates in the file "
+            "configured with configuration parameter 'webserver_verify_ca' "
+            "are spared as they are required for booting the deploy image "
+            "for some boot interfaces."
+        ),
+        'required': True
+    }
+}
+
 
 def _execute_ilo_step(node, step, *args, **kwargs):
     """Executes a particular deploy or clean step.
@@ -381,8 +395,7 @@ class IloManagement(base.ManagementInterface):
         # Reset iLO ejects virtual media
         # Re-create the environment for agent boot, if required
         task.driver.boot.clean_up_ramdisk(task)
-        deploy_opts = deploy_utils.build_agent_options(node)
-        task.driver.boot.prepare_ramdisk(task, deploy_opts)
+        deploy_utils.prepare_agent_boot(task)
 
     @METRICS.timer('IloManagement.reset_ilo_credential')
     @base.deploy_step(priority=0, argsinfo=_RESET_ILO_CREDENTIALS_ARGSINFO)
@@ -643,8 +656,7 @@ class IloManagement(base.ManagementInterface):
         # Firmware might have ejected the virtual media, if it was used.
         # Re-create the environment for agent boot, if required
         task.driver.boot.clean_up_ramdisk(task)
-        deploy_opts = deploy_utils.build_agent_options(node)
-        task.driver.boot.prepare_ramdisk(task, deploy_opts)
+        deploy_utils.prepare_agent_boot(task)
 
         LOG.info("All Firmware update operations completed successfully "
                  "for node: %s.", node.uuid)
@@ -1052,10 +1064,7 @@ class Ilo5Management(IloManagement):
                         'ilo_disk_erase_hdd_check')
                     self._set_driver_internal_value(
                         task, False, 'skip_current_clean_step')
-                    deploy_opts = deploy_utils.build_agent_options(task.node)
-                    task.driver.boot.prepare_ramdisk(task, deploy_opts)
-                    manager_utils.node_power_action(task, states.REBOOT)
-                    return states.CLEANWAIT
+                    return deploy_utils.reboot_to_finish_step(task)
 
                 if not driver_internal_info.get(
                         'ilo_disk_erase_ssd_check') and ('SSD' in disk_types):
@@ -1065,10 +1074,7 @@ class Ilo5Management(IloManagement):
                         'ilo_disk_erase_ssd_check', 'cleaning_reboot')
                     self._set_driver_internal_value(
                         task, False, 'skip_current_clean_step')
-                    deploy_opts = deploy_utils.build_agent_options(task.node)
-                    task.driver.boot.prepare_ramdisk(task, deploy_opts)
-                    manager_utils.node_power_action(task, states.REBOOT)
-                    return states.CLEANWAIT
+                    return deploy_utils.reboot_to_finish_step(task)
 
                 # It will wait until disk erase will complete
                 if self._wait_for_disk_erase_status(task.node):
@@ -1128,3 +1134,51 @@ class Ilo5Management(IloManagement):
                        {'node': task.node.uuid, 'message': ilo_exception})
             manager_utils.cleaning_error_handler(task, log_msg,
                                                  errmsg=ilo_exception)
+
+    @base.clean_step(priority=0, argsinfo=_CLEAR_CA_CERTS_ARGSINFO)
+    def clear_ca_certificates(self, task, certificate_files):
+        """Clears the certificates provided in the list of files to iLO.
+
+        :param task: a task from TaskManager.
+        :param certificate_files: a list of cerificate files.
+        :raises: NodeCleaningFailure, on failure to execute of clean step.
+        :raises: InstanceDeployFailure, on failure to execute of deploy step.
+        """
+        node = task.node
+        driver_internal_info = node.driver_internal_info
+
+        if driver_internal_info.get('clear_ca_certs_flag'):
+            # NOTE(vmud213): Clear the flag and do nothing as this flow
+            # is part of the reboot required by the clean step that is
+            # already executed.
+            driver_internal_info.pop('clear_ca_certs_flag', None)
+            node.driver_internal_info = driver_internal_info
+            node.save()
+            return
+
+        try:
+            ilo_common.clear_certificates(task, certificate_files)
+        except (exception.IloOperationNotSupported,
+                exception.IloOperationError) as ir_exception:
+            msg = (_("Step 'clear_ca_certificates' failed on node %(node)s "
+                     "with error: %(err)s") %
+                   {'node': node.uuid, 'err': ir_exception})
+            if node.clean_step:
+                raise exception.NodeCleaningFailure(msg)
+            raise exception.InstanceDeployFailure(msg)
+
+        driver_internal_info['clear_ca_certs_flag'] = True
+        node.driver_internal_info = driver_internal_info
+        node.save()
+
+        deploy_opts = deploy_utils.build_agent_options(task.node)
+        task.driver.boot.prepare_ramdisk(task, deploy_opts)
+        manager_utils.node_power_action(task, states.REBOOT)
+
+        # set_async_step_flags calls node.save()
+        deploy_utils.set_async_step_flags(
+            node,
+            reboot=True,
+            skip_current_step=False)
+
+        return deploy_utils.get_async_step_return_state(task.node)
