@@ -19,7 +19,6 @@ from ironic_lib import metrics_utils
 from oslo_log import log as logging
 
 from ironic.common import boot_devices
-from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import states
 from ironic.conductor import task_manager
@@ -38,74 +37,6 @@ class PXEBoot(pxe_base.PXEBaseMixin, base.BootInterface):
     capabilities = ['ramdisk_boot', 'pxe_boot']
 
 
-class PXERamdiskDeploy(agent_base.AgentBaseMixin, agent_base.HeartbeatMixin,
-                       base.DeployInterface):
-
-    def get_properties(self):
-        return {}
-
-    def validate(self, task):
-        if 'ramdisk_boot' not in task.driver.boot.capabilities:
-            raise exception.InvalidParameterValue(
-                message=_('Invalid configuration: The boot interface '
-                          'must have the `ramdisk_boot` capability. '
-                          'You are using an incompatible boot interface.'))
-        task.driver.boot.validate(task)
-
-        # Validate node capabilities
-        deploy_utils.validate_capabilities(task.node)
-
-    @METRICS.timer('RamdiskDeploy.deploy')
-    @base.deploy_step(priority=100)
-    @task_manager.require_exclusive_lock
-    def deploy(self, task):
-        if ('configdrive' in task.node.instance_info
-                and 'ramdisk_boot_configdrive' not in
-                task.driver.boot.capabilities):
-            # TODO(dtantsur): make it an actual error?
-            LOG.warning('A configuration drive is present in the ramdisk '
-                        'deployment request of node %(node)s with boot '
-                        'interface %(drv)s. The configuration drive will be '
-                        'ignored for this deployment.',
-                        {'node': task.node, 'drv': task.node.boot_interface})
-        manager_utils.node_power_action(task, states.POWER_OFF)
-        # Tenant neworks must enable connectivity to the boot
-        # location, as reboot() can otherwise be very problematic.
-        # IDEA(TheJulia): Maybe a "trusted environment" mode flag
-        # that we otherwise fail validation on for drivers that
-        # require explicit security postures.
-        with manager_utils.power_state_for_network_configuration(task):
-            task.driver.network.configure_tenant_networks(task)
-
-        # calling boot.prepare_instance will also set the node
-        # to PXE boot, and update PXE templates accordingly
-        task.driver.boot.prepare_instance(task)
-
-        # Power-on the instance, with PXE prepared, we're done.
-        manager_utils.node_power_action(task, states.POWER_ON)
-        LOG.info('Deployment setup for node %s done', task.node.uuid)
-        return None
-
-    @METRICS.timer('RamdiskDeploy.prepare')
-    @task_manager.require_exclusive_lock
-    def prepare(self, task):
-        node = task.node
-
-        deploy_utils.populate_storage_driver_internal_info(task)
-        if node.provision_state == states.DEPLOYING:
-            # Ask the network interface to validate itself so
-            # we can ensure we are able to proceed.
-            task.driver.network.validate(task)
-
-            manager_utils.node_power_action(task, states.POWER_OFF)
-            # NOTE(TheJulia): If this was any other interface, we would
-            # unconfigure tenant networks, add provisioning networks, etc.
-            task.driver.storage.attach_volumes(task)
-        if node.provision_state in (states.ACTIVE, states.UNRESCUING):
-            # In the event of takeover or unrescue.
-            task.driver.boot.prepare_instance(task)
-
-
 class PXEAnacondaDeploy(agent_base.AgentBaseMixin, agent_base.HeartbeatMixin,
                         base.DeployInterface):
 
@@ -147,6 +78,9 @@ class PXEAnacondaDeploy(agent_base.AgentBaseMixin, agent_base.HeartbeatMixin,
             # NOTE(TheJulia): If this was any other interface, we would
             # unconfigure tenant networks, add provisioning networks, etc.
             task.driver.storage.attach_volumes(task)
+            node.instance_info = deploy_utils.build_instance_info_for_deploy(
+                task)
+            node.save()
         if node.provision_state in (states.ACTIVE, states.UNRESCUING):
             # In the event of takeover or unrescue.
             task.driver.boot.prepare_instance(task)
@@ -185,16 +119,16 @@ class PXEAnacondaDeploy(agent_base.AgentBaseMixin, agent_base.HeartbeatMixin,
             agent_base.log_and_raise_deployment_error(task, msg)
 
         try:
+            task.process_event('resume')
             self.clean_up(task)
             manager_utils.node_power_action(task, states.POWER_OFF)
             task.driver.network.remove_provisioning_network(task)
             task.driver.network.configure_tenant_networks(task)
             manager_utils.node_power_action(task, states.POWER_ON)
-            node.provision_state = states.ACTIVE
-            node.save()
+            task.process_event('done')
         except Exception as e:
-            msg = (_('Error rebooting node %(node)s after deploy. '
-                     'Error: %(error)s') %
+            msg = (_('An error occurred after deployment, while preparing to '
+                     'reboot the node %(node)s: %(error)s') %
                    {'node': node.uuid, 'error': e})
             agent_base.log_and_raise_deployment_error(task, msg)
 
@@ -223,3 +157,14 @@ class PXEAnacondaDeploy(agent_base.AgentBaseMixin, agent_base.HeartbeatMixin,
                       '%(agent_status_message)s', msg)
             deploy_utils.set_failed_state(task, agent_status_message,
                                           collect_logs=False)
+
+    @METRICS.timer('AnacondaDeploy.clean_up')
+    @task_manager.require_exclusive_lock
+    def clean_up(self, task):
+        super(PXEAnacondaDeploy, self).clean_up(task)
+        node = task.node
+        # NOTE(rloo): These were added during deployment, as a side-effect of
+        # pxe_utils.get_instance_image_info().
+        node.del_driver_internal_info('stage2')
+        node.del_driver_internal_info('ks_template')
+        node.save()
