@@ -43,6 +43,7 @@ from ironic.common import config as ironic_config
 from ironic.common import context as ironic_context
 from ironic.common import driver_factory
 from ironic.common import hash_ring
+from ironic.common import rpc
 from ironic.common import utils as common_utils
 from ironic.conf import CONF
 from ironic.drivers import base as drivers_base
@@ -55,7 +56,7 @@ logging.setup(CONF, 'ironic')
 
 
 # NOTE(rpittau) this function allows autospec for classmethods and
-# staticmethods in Python 3.6, while no issue occurs in Python 3.7
+# staticmethods in Python 3.6, while no issue occurs in Python 3.7.4
 # and later.
 # For more info please see: http://bugs.python.org/issue23078
 def _patch_mock_callable(obj):
@@ -117,6 +118,8 @@ class TestCase(oslo_test_base.BaseTestCase):
         for factory in driver_factory._INTERFACE_LOADERS.values():
             factory._extension_manager = None
 
+        rpc.set_global_manager(None)
+
         # Ban running external processes via 'execute' like functions. If the
         # patched function is called, an exception is raised to warn the
         # tester.
@@ -133,7 +136,7 @@ class TestCase(oslo_test_base.BaseTestCase):
             # subprocess.Popen is a class
             self.patch(subprocess, 'Popen', DoNotCallPopen)
 
-        if sys.version_info < (3, 7):
+        if sys.version_info < (3, 7, 4):
             _patch_mock_callable._old_func = mock._callable
             mock._callable = _patch_mock_callable
 
@@ -159,7 +162,7 @@ class TestCase(oslo_test_base.BaseTestCase):
                 values = ['fake']
 
             if iface == 'deploy':
-                values.extend(['iscsi', 'direct'])
+                values.extend(['direct', 'anaconda'])
             elif iface == 'boot':
                 values.append('pxe')
             elif iface == 'storage':
