@@ -20,6 +20,7 @@ from unittest import mock
 
 from ironic.common import exception
 from ironic.conductor import task_manager
+from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules.irmc import raid
 from ironic.tests.unit.drivers.modules.irmc import test_common
 
@@ -138,8 +139,10 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
             ]
         }
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter')
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter',
+                autospec=True)
     def test___fail_validation_with_none_raid_adapter_profile(
             self, get_raid_adapter_mock, get_physical_disk_mock):
         get_raid_adapter_mock.return_value = None
@@ -158,8 +161,10 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
                               raid._validate_physical_disks,
                               task.node, target_raid_config['logical_disks'])
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter')
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter',
+                autospec=True)
     def test___fail_validation_without_raid_level(
             self, get_raid_adapter_mock, get_physical_disk_mock):
         get_raid_adapter_mock.return_value = self.raid_adapter_profile
@@ -177,8 +182,10 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
                               raid._validate_physical_disks,
                               task.node, target_raid_config['logical_disks'])
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter')
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter',
+                autospec=True)
     def test___fail_validation_with_raid_level_is_none(self,
                                                        get_raid_adapter_mock,
                                                        get_physical_disk_mock):
@@ -198,8 +205,10 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
                               raid._validate_physical_disks,
                               task.node, target_raid_config['logical_disks'])
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter')
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter',
+                autospec=True)
     def test__fail_validation_without_physical_disks(
             self, get_raid_adapter_mock, get_physical_disk_mock):
         get_raid_adapter_mock.return_value = {
@@ -235,8 +244,10 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
                               raid._validate_physical_disks,
                               task.node, target_raid_config['logical_disks'])
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter')
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter',
+                autospec=True)
     def test___fail_validation_with_raid_level_outside_list(
             self, get_raid_adapter_mock, get_physical_disk_mock):
         get_raid_adapter_mock.return_value = self.raid_adapter_profile
@@ -256,9 +267,12 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
                               task.node, target_raid_config['logical_disks'])
 
     @mock.patch(
-        'ironic.drivers.modules.irmc.raid._validate_logical_drive_capacity')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter')
+        'ironic.drivers.modules.irmc.raid._validate_logical_drive_capacity',
+        autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter',
+                autospec=True)
     def test__fail_validation_with_not_enough_valid_disks(
             self, get_raid_adapter_mock, get_physical_disk_mock,
             capacity_mock):
@@ -282,8 +296,10 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
                               raid._validate_physical_disks,
                               task.node, target_raid_config['logical_disks'])
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter')
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter',
+                autospec=True)
     def test__fail_validation_with_physical_disk_insufficient(
             self, get_raid_adapter_mock, get_physical_disk_mock):
         get_raid_adapter_mock.return_value = self.raid_adapter_profile
@@ -307,8 +323,10 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
                               raid._validate_physical_disks,
                               task.node, target_raid_config['logical_disks'])
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter')
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter',
+                autospec=True)
     def test__fail_validation_with_physical_disk_not_enough_disks(
             self, get_raid_adapter_mock, get_physical_disk_mock):
         get_raid_adapter_mock.return_value = self.raid_adapter_profile
@@ -331,8 +349,10 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
                               raid._validate_physical_disks,
                               task.node, target_raid_config['logical_disks'])
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter')
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter',
+                autospec=True)
     def test__fail_validation_with_physical_disk_incorrect_valid_disks(
             self, get_raid_adapter_mock, get_physical_disk_mock):
         get_raid_adapter_mock.return_value = self.raid_adapter_profile
@@ -358,8 +378,10 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
                               raid._validate_physical_disks,
                               task.node, target_raid_config['logical_disks'])
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter')
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter',
+                autospec=True)
     def test__fail_validation_with_physical_disk_outside_valid_disks_1(
             self, get_raid_adapter_mock, get_physical_disk_mock):
         get_raid_adapter_mock.return_value = self.raid_adapter_profile
@@ -383,9 +405,12 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
                               task.node, target_raid_config['logical_disks'])
 
     @mock.patch(
-        'ironic.drivers.modules.irmc.raid._validate_logical_drive_capacity')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter')
+        'ironic.drivers.modules.irmc.raid._validate_logical_drive_capacity',
+        autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter',
+                autospec=True)
     def test__fail_validation_with_physical_disk_outside_valid_slots_2(
             self, get_raid_adapter_mock, get_physical_disk_mock,
             capacity_mock):
@@ -418,9 +443,12 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
                               task.node, target_raid_config['logical_disks'])
 
     @mock.patch(
-        'ironic.drivers.modules.irmc.raid._validate_logical_drive_capacity')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk')
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter')
+        'ironic.drivers.modules.irmc.raid._validate_logical_drive_capacity',
+        autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_physical_disk',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter',
+                autospec=True)
     def test__fail_validation_with_duplicated_physical_disks(
             self, get_raid_adapter_mock, get_physical_disk_mock,
             capacity_mock):
@@ -452,7 +480,8 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
                               raid._validate_physical_disks,
                               task.node, target_raid_config['logical_disks'])
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter')
+    @mock.patch('ironic.drivers.modules.irmc.raid._get_raid_adapter',
+                autospec=True)
     def test__fail_validation_with_difference_physical_disks_type(
             self, get_raid_adapter_mock):
         get_raid_adapter_mock.return_value = {
@@ -573,11 +602,14 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
                           raid._validate_logical_drive_capacity,
                           disk, self.valid_disk_slots)
 
-    @mock.patch('ironic.common.raid.update_raid_info')
-    @mock.patch('ironic.drivers.modules.irmc.raid.client')
-    def test__commit_raid_config_with_logical_drives(self, client_mock,
-                                                     update_raid_info_mock):
-        client_mock.elcm.get_raid_adapter.return_value = {
+    @mock.patch('ironic.common.raid.update_raid_info', autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid.client.elcm.'
+                'get_raid_adapter', autospec=True)
+    @mock.patch.object(deploy_utils, 'set_async_step_flags', autospec=True)
+    def test__commit_raid_config_with_logical_drives(
+            self, set_async_step_flags_mock,
+            get_raid_adapter_mock, update_raid_info_mock):
+        get_raid_adapter_mock.return_value = {
             "Server": {
                 "HWConfigurationIrmc": {
                     "Adapters": {
@@ -665,10 +697,12 @@ class IRMCRaidConfigurationInternalMethodsTestCase(test_common.BaseIRMCTest):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             raid._commit_raid_config(task)
-            client_mock.elcm.get_raid_adapter.assert_called_once_with(
+            get_raid_adapter_mock.assert_called_once_with(
                 task.node.driver_info)
             update_raid_info_mock.assert_called_once_with(
                 task.node, task.node.raid_config)
+            set_async_step_flags_mock.assert_called_once_with(
+                task.node, reboot=True, skip_current_step=True, polling=True)
             self.assertEqual(task.node.raid_config['logical_disks'],
                              expected_raid_config)
 
@@ -730,9 +764,12 @@ class IRMCRaidConfigurationTestCase(test_common.BaseIRMCTest):
             self.assertRaises(exception.MissingParameterValue,
                               raid_configuration.create_configuration, task)
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._validate_physical_disks')
-    @mock.patch('ironic.drivers.modules.irmc.raid._create_raid_adapter')
-    @mock.patch('ironic.drivers.modules.irmc.raid._commit_raid_config')
+    @mock.patch('ironic.drivers.modules.irmc.raid._validate_physical_disks',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._create_raid_adapter',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._commit_raid_config',
+                autospec=True)
     def test_create_raid_with_raid_1_and_0(self, commit_mock,
                                            create_raid_mock, validation_mock):
         expected_input = {
@@ -759,9 +796,12 @@ class IRMCRaidConfigurationTestCase(test_common.BaseIRMCTest):
                 task.node, expected_input['logical_disks'])
             commit_mock.assert_called_once_with(task)
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._validate_physical_disks')
-    @mock.patch('ironic.drivers.modules.irmc.raid._create_raid_adapter')
-    @mock.patch('ironic.drivers.modules.irmc.raid._commit_raid_config')
+    @mock.patch('ironic.drivers.modules.irmc.raid._validate_physical_disks',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._create_raid_adapter',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.irmc.raid._commit_raid_config',
+                autospec=True)
     def test_create_raid_with_raid_5_and_0(self, commit_mock,
                                            create_raid_mock, validation_mock):
         expected_input = {
@@ -788,7 +828,8 @@ class IRMCRaidConfigurationTestCase(test_common.BaseIRMCTest):
                 task.node, expected_input['logical_disks'])
             commit_mock.assert_called_once_with(task)
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._delete_raid_adapter')
+    @mock.patch('ironic.drivers.modules.irmc.raid._delete_raid_adapter',
+                autospec=True)
     def test_delete_raid_configuration(self, delete_raid_mock):
 
         with task_manager.acquire(self.context, self.node.uuid,
@@ -796,7 +837,8 @@ class IRMCRaidConfigurationTestCase(test_common.BaseIRMCTest):
             task.driver.raid.delete_configuration(task)
             delete_raid_mock.assert_called_once_with(task.node)
 
-    @mock.patch('ironic.drivers.modules.irmc.raid._delete_raid_adapter')
+    @mock.patch('ironic.drivers.modules.irmc.raid._delete_raid_adapter',
+                autospec=True)
     def test_delete_raid_configuration_return_cleared_raid_config(
             self, delete_raid_mock):
         with task_manager.acquire(self.context, self.node.uuid,
