@@ -36,7 +36,6 @@ from ironic.drivers import base as drivers_base
 from ironic.drivers.modules import agent_base
 from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
-from ironic.drivers.modules import fake
 from ironic.drivers.modules import ipxe
 from ironic.drivers.modules import pxe
 from ironic.drivers.modules import pxe_base
@@ -271,7 +270,7 @@ class PXEBootTestCase(db_base.DbTestCase):
                               dhcp_factory_mock,
                               set_boot_device_mock,
                               get_boot_mode_mock,
-                              uefi=False,
+                              uefi=True,
                               cleaning=False,
                               ipxe_use_swift=False,
                               whole_disk_image=False,
@@ -295,6 +294,7 @@ class PXEBootTestCase(db_base.DbTestCase):
         driver_internal_info = self.node.driver_internal_info
         driver_internal_info['is_whole_disk_image'] = whole_disk_image
         self.node.driver_internal_info = driver_internal_info
+
         if mode == 'rescue':
             mock_deploy_img_info.return_value = {
                 'rescue_kernel': 'a',
@@ -356,11 +356,21 @@ class PXEBootTestCase(db_base.DbTestCase):
         self.node.save()
         self._test_prepare_ramdisk()
 
+    def test_prepare_ramdisk_bios(self):
+        self.node.provision_state = states.DEPLOYING
+        self.node.save()
+        self._test_prepare_ramdisk(uefi=True)
+
     def test_prepare_ramdisk_rescue(self):
         self.node.provision_state = states.RESCUING
         self.node.save()
         self._test_prepare_ramdisk(mode='rescue')
 
+    def test_prepare_ramdisk_rescue_bios(self):
+        self.node.provision_state = states.RESCUING
+        self.node.save()
+        self._test_prepare_ramdisk(mode='rescue', uefi=True)
+
     def test_prepare_ramdisk_uefi(self):
         self.node.provision_state = states.DEPLOYING
         self.node.save()
@@ -391,7 +401,8 @@ class PXEBootTestCase(db_base.DbTestCase):
             self, set_boot_mode_mock):
         self.node.provision_state = states.DEPLOYING
         self.node.save()
-        self._test_prepare_ramdisk(node_boot_mode=boot_modes.LEGACY_BIOS)
+        self._test_prepare_ramdisk(node_boot_mode=boot_modes.LEGACY_BIOS,
+                                   uefi=False)
 
         with task_manager.acquire(self.context, self.node.uuid) as task:
             driver_internal_info = task.node.driver_internal_info
@@ -408,8 +419,7 @@ class PXEBootTestCase(db_base.DbTestCase):
 
         self.config(default_boot_mode=boot_modes.LEGACY_BIOS, group='deploy')
 
-        self._test_prepare_ramdisk()
-
+        self._test_prepare_ramdisk(uefi=False)
         with task_manager.acquire(self.context, self.node.uuid) as task:
             driver_internal_info = task.node.driver_internal_info
             self.assertIn('deploy_boot_mode', driver_internal_info)
@@ -470,7 +480,7 @@ class PXEBootTestCase(db_base.DbTestCase):
         properties['capabilities'] = 'boot_mode:uefi'
         self.node.properties = properties
         self.node.save()
-        self._test_prepare_ramdisk(uefi=True, node_boot_mode=boot_modes.UEFI)
+        self._test_prepare_ramdisk(node_boot_mode=boot_modes.UEFI)
         self.assertEqual(set_boot_mode_mock.call_count, 0)
 
     @mock.patch.object(pxe_utils, 'clean_up_pxe_env', autospec=True)
@@ -504,7 +514,7 @@ class PXEBootTestCase(db_base.DbTestCase):
     @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
     @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
     @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_prepare_instance_netboot(
+    def test_prepare_instance_netboot_bios(
             self, get_image_info_mock, cache_mock,
             dhcp_factory_mock, switch_pxe_config_mock,
             set_boot_device_mock):
@@ -514,18 +524,19 @@ class PXEBootTestCase(db_base.DbTestCase):
                       'ramdisk': ('', '/path/to/ramdisk')}
         get_image_info_mock.return_value = image_info
         with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.node.properties['capabilities'] = 'boot_mode:bios'
+            task.node.driver_internal_info['root_uuid_or_disk_id'] = (
+                "30212642-09d3-467f-8e09-21685826ab50")
+            task.node.driver_internal_info['is_whole_disk_image'] = False
+            task.node.instance_info = {
+                'capabilities': {'boot_option': 'netboot',
+                                 'boot_mode': 'bios'}}
             dhcp_opts = pxe_utils.dhcp_options_for_instance(
                 task, ipxe_enabled=False, ip_version=4)
             dhcp_opts += pxe_utils.dhcp_options_for_instance(
                 task, ipxe_enabled=False, ip_version=6)
             pxe_config_path = pxe_utils.get_pxe_config_file_path(
                 task.node.uuid)
-            task.node.properties['capabilities'] = 'boot_mode:bios'
-            task.node.driver_internal_info['root_uuid_or_disk_id'] = (
-                "30212642-09d3-467f-8e09-21685826ab50")
-            task.node.driver_internal_info['is_whole_disk_image'] = False
-            task.node.instance_info = {
-                'capabilities': {'boot_option': 'netboot'}}
             task.driver.boot.prepare_instance(task)
 
             get_image_info_mock.assert_called_once_with(
@@ -541,6 +552,47 @@ class PXEBootTestCase(db_base.DbTestCase):
                                                          boot_devices.PXE,
                                                          persistent=True)
 
+    @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
+    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
+    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
+    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
+    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
+    def test_prepare_instance_netboot_uefi(
+            self, get_image_info_mock, cache_mock,
+            dhcp_factory_mock, switch_pxe_config_mock,
+            set_boot_device_mock):
+        provider_mock = mock.MagicMock()
+        dhcp_factory_mock.return_value = provider_mock
+        image_info = {'kernel': ('', '/path/to/kernel'),
+                      'ramdisk': ('', '/path/to/ramdisk')}
+        get_image_info_mock.return_value = image_info
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.node.driver_internal_info['root_uuid_or_disk_id'] = (
+                "30212642-09d3-467f-8e09-21685826ab50")
+            task.node.driver_internal_info['is_whole_disk_image'] = False
+            task.node.instance_info = {
+                'capabilities': {'boot_option': 'netboot'}}
+            dhcp_opts = pxe_utils.dhcp_options_for_instance(
+                task, ipxe_enabled=False, ip_version=4)
+            dhcp_opts += pxe_utils.dhcp_options_for_instance(
+                task, ipxe_enabled=False, ip_version=6)
+            pxe_config_path = pxe_utils.get_pxe_config_file_path(
+                task.node.uuid)
+            task.driver.boot.prepare_instance(task)
+
+            get_image_info_mock.assert_called_once_with(
+                task, ipxe_enabled=False)
+            cache_mock.assert_called_once_with(
+                task, image_info, ipxe_enabled=False)
+            provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
+            switch_pxe_config_mock.assert_called_once_with(
+                pxe_config_path, "30212642-09d3-467f-8e09-21685826ab50",
+                'uefi', False, False, False, False, ipxe_enabled=False,
+                anaconda_boot=False)
+            set_boot_device_mock.assert_called_once_with(task,
+                                                         boot_devices.PXE,
+                                                         persistent=True)
+
     @mock.patch('os.path.isfile', return_value=False, autospec=True)
     @mock.patch.object(pxe_utils, 'create_pxe_config', autospec=True)
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
@@ -561,18 +613,18 @@ class PXEBootTestCase(db_base.DbTestCase):
         self.node.provision_state = states.ACTIVE
         self.node.save()
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            dhcp_opts = pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False)
-            dhcp_opts += pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False, ip_version=6)
-            pxe_config_path = pxe_utils.get_pxe_config_file_path(
-                task.node.uuid)
             task.node.properties['capabilities'] = 'boot_mode:bios'
             task.node.driver_internal_info['root_uuid_or_disk_id'] = (
                 "30212642-09d3-467f-8e09-21685826ab50")
             task.node.driver_internal_info['is_whole_disk_image'] = False
             task.node.instance_info['capabilities'] = instance_info
             task.driver.boot.prepare_instance(task)
+            dhcp_opts = pxe_utils.dhcp_options_for_instance(
+                task, ipxe_enabled=False)
+            dhcp_opts += pxe_utils.dhcp_options_for_instance(
+                task, ipxe_enabled=False, ip_version=6)
+            pxe_config_path = pxe_utils.get_pxe_config_file_path(
+                task.node.uuid)
 
             get_image_info_mock.assert_called_once_with(
                 task, ipxe_enabled=False)
@@ -604,13 +656,13 @@ class PXEBootTestCase(db_base.DbTestCase):
         instance_info = {"boot_option": "netboot"}
         get_image_info_mock.return_value = image_info
         with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.node.properties['capabilities'] = 'boot_mode:bios'
+            task.node.instance_info['capabilities'] = instance_info
+            task.node.driver_internal_info['is_whole_disk_image'] = False
             dhcp_opts = pxe_utils.dhcp_options_for_instance(
                 task, ipxe_enabled=False)
             dhcp_opts += pxe_utils.dhcp_options_for_instance(
                 task, ipxe_enabled=False, ip_version=6)
-            task.node.properties['capabilities'] = 'boot_mode:bios'
-            task.node.instance_info['capabilities'] = instance_info
-            task.node.driver_internal_info['is_whole_disk_image'] = False
 
             task.driver.boot.prepare_instance(task)
 
@@ -637,13 +689,13 @@ class PXEBootTestCase(db_base.DbTestCase):
         get_image_info_mock.return_value = {}
         instance_info = {"boot_option": "netboot"}
         with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.node.properties['capabilities'] = 'boot_mode:bios'
+            task.node.instance_info['capabilities'] = instance_info
+            task.node.driver_internal_info['is_whole_disk_image'] = True
             dhcp_opts = pxe_utils.dhcp_options_for_instance(
                 task, ipxe_enabled=False)
             dhcp_opts += pxe_utils.dhcp_options_for_instance(
                 task, ipxe_enabled=False, ip_version=6)
-            task.node.properties['capabilities'] = 'boot_mode:bios'
-            task.node.instance_info['capabilities'] = instance_info
-            task.node.driver_internal_info['is_whole_disk_image'] = True
             task.driver.boot.prepare_instance(task)
             get_image_info_mock.assert_called_once_with(task,
                                                         ipxe_enabled=False)
@@ -702,7 +754,8 @@ class PXEBootTestCase(db_base.DbTestCase):
             self, get_image_info_mock, cache_mock,
             dhcp_factory_mock, create_pxe_config_mock,
             switch_pxe_config_mock,
-            set_boot_device_mock, config_file_exits=False):
+            set_boot_device_mock, config_file_exits=False,
+            uefi=True):
         image_info = {'kernel': ['', '/path/to/kernel'],
                       'ramdisk': ['', '/path/to/ramdisk']}
         get_image_info_mock.return_value = image_info
@@ -731,12 +784,22 @@ class PXEBootTestCase(db_base.DbTestCase):
             if config_file_exits:
                 self.assertFalse(create_pxe_config_mock.called)
             else:
-                create_pxe_config_mock.assert_called_once_with(
-                    task, mock.ANY, CONF.pxe.pxe_config_template,
-                    ipxe_enabled=False)
+                if not uefi:
+                    create_pxe_config_mock.assert_called_once_with(
+                        task, mock.ANY, CONF.pxe.pxe_config_template,
+                        ipxe_enabled=False)
+                else:
+                    create_pxe_config_mock.assert_called_once_with(
+                        task, mock.ANY, CONF.pxe.uefi_pxe_config_template,
+                        ipxe_enabled=False)
+            if uefi:
+                boot_mode = 'uefi'
+            else:
+                boot_mode = 'bios'
+
             switch_pxe_config_mock.assert_called_once_with(
                 pxe_config_path, None,
-                'bios', False, ipxe_enabled=False, iscsi_boot=False,
+                boot_mode, False, ipxe_enabled=False, iscsi_boot=False,
                 ramdisk_boot=True, anaconda_boot=False)
             set_boot_device_mock.assert_called_once_with(task,
                                                          boot_devices.PXE,
@@ -778,6 +841,70 @@ class PXEBootTestCase(db_base.DbTestCase):
         dhcp_factory_mock.return_value = provider_mock
         self.node.provision_state = states.DEPLOYING
         self.config(http_url='http://fake_url', group='deploy')
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            dhcp_opts = pxe_utils.dhcp_options_for_instance(
+                task, ipxe_enabled=False)
+            dhcp_opts += pxe_utils.dhcp_options_for_instance(
+                task, ipxe_enabled=False, ip_version=6)
+            pxe_config_path = pxe_utils.get_pxe_config_file_path(
+                task.node.uuid)
+
+            task.driver.boot.prepare_instance(task)
+
+            get_image_info_mock.assert_called_once_with(task,
+                                                        ipxe_enabled=False)
+            cache_mock.assert_called_once_with(
+                task, image_info, False)
+            if os.path.isfile('/usr/bin/ksvalidator'):
+                exec_mock.assert_called_once_with(
+                    'ksvalidator', mock.ANY, check_on_exit=[0], attempts=1
+                )
+            provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
+            render_mock.assert_called()
+            write_file_mock.assert_called_with(
+                '/path/to/ks.cfg', render_mock.return_value
+            )
+            create_pxe_config_mock.assert_called_once_with(
+                task, mock.ANY, CONF.pxe.uefi_pxe_config_template,
+                ipxe_enabled=False)
+            switch_pxe_config_mock.assert_called_once_with(
+                pxe_config_path, None,
+                'uefi', False, ipxe_enabled=False, iscsi_boot=False,
+                ramdisk_boot=False, anaconda_boot=True)
+            set_boot_device_mock.assert_called_once_with(task,
+                                                         boot_devices.PXE,
+                                                         persistent=True)
+
+    @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
+    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
+    @mock.patch.object(pxe_utils, 'create_pxe_config', autospec=True)
+    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
+    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
+    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
+    @mock.patch('ironic.drivers.modules.deploy_utils.get_boot_option',
+                return_value='kickstart', autospec=True)
+    @mock.patch('ironic.drivers.modules.deploy_utils.get_ironic_api_url',
+                return_value='http://fakeserver/api', autospec=True)
+    @mock.patch('ironic.common.utils.render_template', autospec=True)
+    @mock.patch('ironic.common.utils.write_to_file', autospec=True)
+    @mock.patch('ironic.common.utils.execute', autospec=True)
+    def test_prepare_instance_kickstart_bios(
+            self, exec_mock, write_file_mock, render_mock, api_url_mock,
+            boot_opt_mock, get_image_info_mock, cache_mock, dhcp_factory_mock,
+            create_pxe_config_mock, switch_pxe_config_mock,
+            set_boot_device_mock):
+        image_info = {'kernel': ['ins_kernel_id', '/path/to/kernel'],
+                      'ramdisk': ['ins_ramdisk_id', '/path/to/ramdisk'],
+                      'stage2': ['ins_stage2_id', '/path/to/stage2'],
+                      'ks_cfg': ['', '/path/to/ks.cfg'],
+                      'ks_template': ['template_id', '/path/to/ks_template']}
+        get_image_info_mock.return_value = image_info
+        provider_mock = mock.MagicMock()
+        dhcp_factory_mock.return_value = provider_mock
+        self.node.provision_state = states.DEPLOYING
+        self.config(http_url='http://fake_url', group='deploy')
+        self.config(default_boot_mode='bios', group='deploy')
+
         with task_manager.acquire(self.context, self.node.uuid) as task:
             dhcp_opts = pxe_utils.dhcp_options_for_instance(
                 task, ipxe_enabled=False)
@@ -831,269 +958,6 @@ class PXEBootTestCase(db_base.DbTestCase):
             secure_boot_mock.assert_called_once_with(task)
 
 
-class PXERamdiskDeployTestCase(db_base.DbTestCase):
-
-    def setUp(self):
-        super(PXERamdiskDeployTestCase, self).setUp()
-        self.temp_dir = tempfile.mkdtemp()
-        self.config(tftp_root=self.temp_dir, group='pxe')
-        self.temp_dir = tempfile.mkdtemp()
-        self.config(images_path=self.temp_dir, group='pxe')
-        self.config(enabled_deploy_interfaces=['ramdisk'])
-        self.config(enabled_boot_interfaces=['pxe'])
-        for iface in drivers_base.ALL_INTERFACES:
-            impl = 'fake'
-            if iface == 'network':
-                impl = 'noop'
-            if iface == 'deploy':
-                impl = 'ramdisk'
-            if iface == 'boot':
-                impl = 'pxe'
-            config_kwarg = {'enabled_%s_interfaces' % iface: [impl],
-                            'default_%s_interface' % iface: impl}
-            self.config(**config_kwarg)
-        self.config(enabled_hardware_types=['fake-hardware'])
-        instance_info = {'kernel': 'kernelUUID',
-                         'ramdisk': 'ramdiskUUID'}
-        self.node = obj_utils.create_test_node(
-            self.context,
-            driver='fake-hardware',
-            instance_info=instance_info,
-            driver_info=DRV_INFO_DICT,
-            driver_internal_info=DRV_INTERNAL_INFO_DICT)
-        self.port = obj_utils.create_test_port(self.context,
-                                               node_id=self.node.id)
-
-    @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
-    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
-    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
-    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
-    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_prepare_instance_ramdisk(
-            self, get_image_info_mock, cache_mock,
-            dhcp_factory_mock, switch_pxe_config_mock,
-            set_boot_device_mock):
-        provider_mock = mock.MagicMock()
-        dhcp_factory_mock.return_value = provider_mock
-        self.node.provision_state = states.DEPLOYING
-        image_info = {'kernel': ('', '/path/to/kernel'),
-                      'ramdisk': ('', '/path/to/ramdisk')}
-        get_image_info_mock.return_value = image_info
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            dhcp_opts = pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False)
-            dhcp_opts += pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False, ip_version=6)
-            pxe_config_path = pxe_utils.get_pxe_config_file_path(
-                task.node.uuid)
-            task.node.properties['capabilities'] = 'boot_option:netboot'
-            task.node.driver_internal_info['is_whole_disk_image'] = False
-            task.driver.deploy.prepare(task)
-            task.driver.deploy.deploy(task)
-
-            get_image_info_mock.assert_called_once_with(task,
-                                                        ipxe_enabled=False)
-            cache_mock.assert_called_once_with(
-                task, image_info, ipxe_enabled=False)
-            provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
-            switch_pxe_config_mock.assert_called_once_with(
-                pxe_config_path, None,
-                'bios', False, ipxe_enabled=False, iscsi_boot=False,
-                ramdisk_boot=True, anaconda_boot=False)
-            set_boot_device_mock.assert_called_once_with(task,
-                                                         boot_devices.PXE,
-                                                         persistent=True)
-
-    @mock.patch.object(pxe.LOG, 'warning', autospec=True)
-    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
-    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
-    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
-    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_deploy(self, mock_image_info, mock_cache,
-                    mock_dhcp_factory, mock_switch_config, mock_warning):
-        image_info = {'kernel': ('', '/path/to/kernel'),
-                      'ramdisk': ('', '/path/to/ramdisk')}
-        mock_image_info.return_value = image_info
-        i_info = self.node.instance_info
-        i_info.update({'capabilities': {'boot_option': 'ramdisk'}})
-        self.node.instance_info = i_info
-        self.node.save()
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            self.assertIsNone(task.driver.deploy.deploy(task))
-            mock_image_info.assert_called_once_with(task, ipxe_enabled=False)
-            mock_cache.assert_called_once_with(
-                task, image_info, ipxe_enabled=False)
-            self.assertFalse(mock_warning.called)
-        i_info['configdrive'] = 'meow'
-        self.node.instance_info = i_info
-        self.node.save()
-        mock_warning.reset_mock()
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            self.assertIsNone(task.driver.deploy.deploy(task))
-            self.assertTrue(mock_warning.called)
-
-    @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
-    def test_prepare(self, mock_prepare_instance):
-        node = self.node
-        node.provision_state = states.DEPLOYING
-        node.instance_info = {}
-        node.save()
-        with task_manager.acquire(self.context, node.uuid) as task:
-            task.driver.deploy.prepare(task)
-            self.assertFalse(mock_prepare_instance.called)
-
-    @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
-    def test_prepare_active(self, mock_prepare_instance):
-        node = self.node
-        node.provision_state = states.ACTIVE
-        node.save()
-        with task_manager.acquire(self.context, node.uuid) as task:
-            task.driver.deploy.prepare(task)
-            mock_prepare_instance.assert_called_once_with(mock.ANY, task)
-
-    @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
-    def test_prepare_unrescuing(self, mock_prepare_instance):
-        node = self.node
-        node.provision_state = states.UNRESCUING
-        node.save()
-        with task_manager.acquire(self.context, node.uuid) as task:
-            task.driver.deploy.prepare(task)
-            mock_prepare_instance.assert_called_once_with(mock.ANY, task)
-
-    @mock.patch.object(deploy_utils, 'validate_image_properties',
-                       autospec=True)
-    def test_validate(self, mock_validate_img):
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.driver.deploy.validate(task)
-        self.assertTrue(mock_validate_img.called)
-
-    @mock.patch.object(deploy_utils, 'validate_image_properties',
-                       autospec=True)
-    def test_validate_with_boot_iso(self, mock_validate_img):
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.node.instance_info = {
-                'boot_iso': 'isoUUID'
-            }
-            task.driver.deploy.validate(task)
-        self.assertTrue(mock_validate_img.called)
-
-    @mock.patch.object(fake.FakeBoot, 'validate', autospec=True)
-    @mock.patch.object(deploy_utils, 'validate_image_properties',
-                       autospec=True)
-    def test_validate_interface_mismatch(self, mock_validate_image,
-                                         mock_boot_validate):
-        node = self.node
-        node.boot_interface = 'fake'
-        node.save()
-        self.config(enabled_boot_interfaces=['fake'],
-                    default_boot_interface='fake')
-        with task_manager.acquire(self.context, node.uuid) as task:
-            error = self.assertRaises(exception.InvalidParameterValue,
-                                      task.driver.deploy.validate, task)
-            error_message = ('Invalid configuration: The boot interface must '
-                             'have the `ramdisk_boot` capability. You are '
-                             'using an incompatible boot interface.')
-            self.assertEqual(error_message, str(error))
-            self.assertFalse(mock_boot_validate.called)
-            self.assertFalse(mock_validate_image.called)
-
-    @mock.patch.object(pxe.PXEBoot, 'validate', autospec=True)
-    def test_validate_calls_boot_validate(self, mock_validate):
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.driver.deploy.validate(task)
-            mock_validate.assert_called_once_with(mock.ANY, task)
-
-    @mock.patch.object(manager_utils, 'restore_power_state_if_needed',
-                       autospec=True)
-    @mock.patch.object(manager_utils, 'power_on_node_if_needed',
-                       autospec=True)
-    @mock.patch.object(pxe.LOG, 'warning', autospec=True)
-    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
-    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
-    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
-    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_deploy_with_smartnic_port(
-            self, mock_image_info, mock_cache,
-            mock_dhcp_factory, mock_switch_config, mock_warning,
-            power_on_node_if_needed_mock, restore_power_state_mock):
-        image_info = {'kernel': ('', '/path/to/kernel'),
-                      'ramdisk': ('', '/path/to/ramdisk')}
-        mock_image_info.return_value = image_info
-        i_info = self.node.instance_info
-        i_info.update({'capabilities': {'boot_option': 'ramdisk'}})
-        self.node.instance_info = i_info
-        self.node.save()
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            power_on_node_if_needed_mock.return_value = states.POWER_OFF
-            self.assertIsNone(task.driver.deploy.deploy(task))
-            mock_image_info.assert_called_once_with(task, ipxe_enabled=False)
-            mock_cache.assert_called_once_with(
-                task, image_info, ipxe_enabled=False)
-            self.assertFalse(mock_warning.called)
-            power_on_node_if_needed_mock.assert_called_once_with(task)
-            restore_power_state_mock.assert_called_once_with(
-                task, states.POWER_OFF)
-        i_info['configdrive'] = 'meow'
-        self.node.instance_info = i_info
-        self.node.save()
-        mock_warning.reset_mock()
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            self.assertIsNone(task.driver.deploy.deploy(task))
-            self.assertTrue(mock_warning.called)
-
-    @mock.patch.object(agent_base, 'get_steps', autospec=True)
-    def test_get_clean_steps(self, mock_get_steps):
-        # Test getting clean steps
-        mock_steps = [{'priority': 10, 'interface': 'deploy',
-                       'step': 'erase_devices'}]
-        mock_get_steps.return_value = mock_steps
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            steps = task.driver.deploy.get_clean_steps(task)
-            mock_get_steps.assert_called_once_with(
-                task, 'clean', interface='deploy',
-                override_priorities={'erase_devices': None,
-                                     'erase_devices_metadata': None})
-        self.assertEqual(mock_steps, steps)
-
-    def test_get_deploy_steps(self):
-        # Only the default deploy step exists in the ramdisk deploy
-        expected = [{'argsinfo': None, 'interface': 'deploy', 'priority': 100,
-                     'step': 'deploy'}]
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            steps = task.driver.deploy.get_deploy_steps(task)
-            self.assertEqual(expected, steps)
-
-    @mock.patch.object(agent_base, 'execute_step', autospec=True)
-    def test_execute_clean_step(self, mock_execute_step):
-        step = {
-            'priority': 10,
-            'interface': 'deploy',
-            'step': 'erase_devices',
-            'reboot_requested': False
-        }
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            result = task.driver.deploy.execute_clean_step(task, step)
-            self.assertIs(result, mock_execute_step.return_value)
-            mock_execute_step.assert_called_once_with(task, step, 'clean')
-
-    @mock.patch.object(deploy_utils, 'prepare_inband_cleaning', autospec=True)
-    def test_prepare_cleaning(self, prepare_inband_cleaning_mock):
-        prepare_inband_cleaning_mock.return_value = states.CLEANWAIT
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            self.assertEqual(
-                states.CLEANWAIT, task.driver.deploy.prepare_cleaning(task))
-            prepare_inband_cleaning_mock.assert_called_once_with(
-                task, manage_boot=True)
-
-    @mock.patch.object(deploy_utils, 'tear_down_inband_cleaning',
-                       autospec=True)
-    def test_tear_down_cleaning(self, tear_down_cleaning_mock):
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.driver.deploy.tear_down_cleaning(task)
-            tear_down_cleaning_mock.assert_called_once_with(
-                task, manage_boot=True)
-
-
 class PXEAnacondaDeployTestCase(db_base.DbTestCase):
 
     def setUp(self):
@@ -1158,15 +1022,23 @@ class PXEAnacondaDeployTestCase(db_base.DbTestCase):
             mock_prepare_ks_config.assert_called_once_with(task, image_info,
                                                            anaconda_boot=True)
 
+    @mock.patch.object(deploy_utils, 'build_instance_info_for_deploy',
+                       autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
-    def test_prepare(self, mock_prepare_instance):
+    def test_prepare(self, mock_prepare_instance, mock_build_instance):
+
         node = self.node
         node.provision_state = states.DEPLOYING
         node.instance_info = {}
         node.save()
+        updated_instance_info = {'image_url': 'foo'}
+        mock_build_instance.return_value = updated_instance_info
         with task_manager.acquire(self.context, node.uuid) as task:
             task.driver.deploy.prepare(task)
             self.assertFalse(mock_prepare_instance.called)
+            mock_build_instance.assert_called_once_with(task)
+            node.refresh()
+            self.assertEqual(updated_instance_info, node.instance_info)
 
     @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
     def test_prepare_active(self, mock_prepare_instance):
@@ -1325,6 +1197,7 @@ class PXEValidateRescueTestCase(db_base.DbTestCase):
 class PXEBootRetryTestCase(db_base.DbTestCase):
 
     boot_interface = 'pxe'
+    boot_interface_class = pxe.PXEBoot
 
     def setUp(self):
         super(PXEBootRetryTestCase, self).setUp()
@@ -1336,31 +1209,28 @@ class PXEBootRetryTestCase(db_base.DbTestCase):
             boot_interface=self.boot_interface,
             provision_state=states.DEPLOYWAIT)
 
-    @mock.patch.object(pxe.PXEBoot, '_check_boot_status', autospec=True)
-    def test_check_boot_timeouts(self, mock_check_status, mock_power,
-                                 mock_boot_dev):
+    def test_check_boot_timeouts(self, mock_power, mock_boot_dev):
         def _side_effect(iface, task):
             self.assertEqual(self.node.uuid, task.node.uuid)
 
-        mock_check_status.side_effect = _side_effect
+        fake_node = obj_utils.create_test_node(
+            self.context,
+            uuid=uuidutils.generate_uuid(),
+            driver='fake-hardware',
+            boot_interface='fake',
+            provision_state=states.DEPLOYWAIT)
+
         manager = mock.Mock(spec=['iter_nodes'])
         manager.iter_nodes.return_value = [
-            (uuidutils.generate_uuid(), 'fake-hardware', ''),
-            (self.node.uuid, self.node.driver, self.node.conductor_group)
+            (fake_node.uuid, 'fake-hardware', ''),
+            (self.node.uuid, self.node.driver, self.node.conductor_group),
         ]
-        iface = pxe.PXEBoot()
-        iface._check_boot_timeouts(manager, self.context)
-        mock_check_status.assert_called_once_with(iface, mock.ANY)
-
-    def test_check_boot_status_another_boot_interface(self, mock_power,
-                                                      mock_boot_dev):
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            task.driver.boot = fake.FakeBoot()
-            pxe.PXEBoot()._check_boot_status(task)
-            self.assertTrue(task.shared)
-        self.assertFalse(mock_power.called)
-        self.assertFalse(mock_boot_dev.called)
+        with mock.patch.object(self.boot_interface_class, '_check_boot_status',
+                               autospec=True) as mock_check_status:
+            mock_check_status.side_effect = _side_effect
+            iface = self.boot_interface_class()
+            iface._check_boot_timeouts(manager, self.context)
+            mock_check_status.assert_called_once_with(iface, mock.ANY)
 
     def test_check_boot_status_recent_power_change(self, mock_power,
                                                    mock_boot_dev):
@@ -1411,3 +1281,4 @@ class PXEBootRetryTestCase(db_base.DbTestCase):
 class iPXEBootRetryTestCase(PXEBootRetryTestCase):
 
     boot_interface = 'ipxe'
+    boot_interface_class = ipxe.iPXEBoot
